import asyncio
import thriftpy

from aiothrift.server import make_server

pingpong_thrift = thriftpy.load('pingpong.thrift', module_name='pingpong_thrift')


class Dispatcher:
    def ping(self):
        return "pong"

    async def add(self, a, b):
        await asyncio.sleep(1)
        return a + b


loop = asyncio.get_event_loop()

server = loop.run_until_complete(
    make_server(pingpong_thrift.PingPong, Dispatcher(), '127.0.0.1', 6000, loop=loop))

try:
    loop.run_forever()
except KeyboardInterrupt:
    pass

server.close()
loop.run_until_complete(server.wait_closed())
loop.close()



import thriftpy
import asyncio
import aiothrift

pingpong_thrift = thriftpy.load('pingpong.thrift', module_name='pingpong_thrift')

loop = asyncio.get_event_loop()


async def go():
    conn = await aiothrift.create_connection(pingpong_thrift.PingPong, ('127.0.0.1', 6000), loop=loop, timeout=2)
    print(await conn.ping())
    print(await conn.add(5, 6))
    conn.close()


loop.run_until_complete(go())
loop.close()



import asyncio
import sys

from asyncio.base_events import BaseEventLoop

PY_35 = sys.version_info >= (3, 5)

if hasattr(asyncio, 'ensure_future'):
    async_task = asyncio.ensure_future
else:
    async_task = asyncio.async  # Deprecated since 3.4.4

# create_future is new in version 3.5.2
if hasattr(BaseEventLoop, 'create_future'):
    def create_future(loop):
        return loop.create_future()
else:
    def create_future(loop):
        return asyncio.Future(loop=loop)


def args2kwargs(thrift_spec, *args):
    arg_names = [item[1][1] for item in sorted(thrift_spec.items())]
    return dict(zip(arg_names, args))



import asyncio
import async_timeout

import functools

from thriftpy.thrift import TMessageType, TApplicationException

from .protocol import TBinaryProtocol
from .util import args2kwargs
from .errors import ConnectionClosedError


@asyncio.coroutine
def create_connection(service, address, *, protocol_cls=TBinaryProtocol,
                      timeout, loop=None):
    host, port = address
    reader, writer = yield from asyncio.open_connection(
        host, port, loop=loop)
    sock = writer.transport.get_extra_info('socket')
    address = sock.getpeername()
    address = tuple(address[:2])
    iprotocol = protocol_cls(reader)
    oprotocol = protocol_cls(writer)

    return ThriftConnection(service, iprot=iprotocol, oprot=oprotocol,
                            address=address, loop=loop, timeout=timeout)


class ThriftConnection:
    def __init__(self, service, *, iprot, oprot, address, loop=None, timeout=None):
        self.service = service
        self._reader = iprot.trans
        self._writer = oprot.trans
        self.loop = loop
        self.timeout = timeout
        self.address = address
        self.closed = False
        self._oprot = oprot
        self._iprot = iprot
        self._seqid = 0
        self._init_rpc_apis()

    def _init_rpc_apis(self):
        for api in self.service.thrift_services:
            setattr(self, api, functools.partial(self._send_call, api))

    @asyncio.coroutine
    def _send_call(self, api, *args, **kwargs):
        with async_timeout.timeout(self.timeout):
            if self._reader is None or self._reader.at_eof():
                raise ConnectionClosedError('Connection closed')

            _kw = args2kwargs(getattr(self.service, api + "_args").thrift_spec,
                              *args)
            kwargs.update(_kw)
            result_cls = getattr(self.service, api + "_result")

            self._oprot.write_message_begin(api, TMessageType.CALL, self._seqid)
            args = getattr(self.service, api + '_args')()
            for k, v in kwargs.items():
                setattr(args, k, v)
            args.write(self._oprot)
            self._oprot.write_message_end()
            yield from self._oprot.trans.drain()

            # writer.write
            # wait result only if non-oneway
            if not getattr(result_cls, "oneway"):
                result = yield from self._recv(api)
                return result

    @asyncio.coroutine
    def _recv(self, api):
        fname, mtype, rseqid = yield from self._iprot.read_message_begin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            yield from self._iprot.read_struct(x)
            yield from self._iprot.read_message_end()
            raise x
        result = getattr(self.service, api + '_result')()
        yield from self._iprot.read_struct(result)
        yield from self._iprot.read_message_end()

        if hasattr(result, "success") and result.success is not None:
            return result.success

        # void api without throws
        if len(result.thrift_spec) == 0:
            return

        # check throws
        for k, v in result.__dict__.items():
            if k != "success" and v:
                raise v
        if hasattr(result, 'success'):
            raise TApplicationException(TApplicationException.MISSING_RESULT)

    def close(self):
        self._writer.close()
        self.closed = True



import asyncio

from thriftpy.transport import TTransportException

from aiothrift.processor import TProcessor
from aiothrift.protocol import TBinaryProtocol


class Server:
    def __init__(self, processor, protocol_cls=TBinaryProtocol):
        self.processor = processor
        self.protocol_cls = protocol_cls

    @asyncio.coroutine
    def __call__(self, reader, writer):
        iproto = self.protocol_cls(reader)
        oproto = self.protocol_cls(writer)
        while not reader.at_eof():
            try:
                yield from self.processor.process(iproto, oproto)
            except TTransportException:
                pass
            except Exception:
                pass
        writer.close()


@asyncio.coroutine
def make_server(service, handler,
                host="localhost", port=9090,
                loop=None,
                protocol_cls=TBinaryProtocol,
                ):
    processor = TProcessor(service, handler)
    if loop is None:
        loop = asyncio.get_event_loop()
    server = yield from asyncio.start_server(
        Server(processor, protocol_cls), host, port, loop=loop)
    return server



import pathlib
import re
from setuptools import setup

here = pathlib.Path(__file__).parent
fname = here / 'aiothrift' / '__init__.py'

with fname.open() as fp:
    try:
        version = re.findall(r"__version__ = '([^']+)'$", fp.read(), re.M)[0]
    except IndexError:
        raise RuntimeError('Unable to determin version.')


def read(name):
    fname = here / name
    with fname.open() as f:
        return f.read()


with open('requirements.txt') as f:
    REQUIREMENTS = list(map(lambda l: l.strip(), f.readlines()))

setup(name='aiothrift',
      version=version,
      description='async thrift server and client',
      long_description='\n\n'.join([read('README.rst'), read('CHANGES.rst')]),
      classifiers=[

          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'Topic :: Software Development',
          'Topic :: Software Development :: Libraries',
      ],
      platforms=['POSIX', 'WINDOWS'],
      url='http://github.com/moonshadow/aiothrift/',
      author='Wang Haowei',
      author_email='hwwangwang@gmail.com',
      license='MIT',
      packages=['aiothrift'],
      install_requires=REQUIREMENTS,
      include_package_data=True,
      )



import asyncio

from thriftpy.thrift import TType, TApplicationException, TMessageType


class TProcessor(object):
    """Base class for procsessor, which works on two streams."""

    def __init__(self, service, handler):
        self._service = service
        self._handler = handler

    @asyncio.coroutine
    def process_in(self, iprot):
        api, type, seqid = yield from iprot.read_message_begin()
        if api not in self._service.thrift_services:
            yield from iprot.skip(TType.STRUCT)
            yield from iprot.read_message_end()
            return api, seqid, TApplicationException(TApplicationException.UNKNOWN_METHOD), None  # noqa

        args = getattr(self._service, api + "_args")()
        yield from iprot.read_struct(args)
        yield from iprot.read_message_end()
        result = getattr(self._service, api + "_result")()

        # convert kwargs to args
        api_args = [args.thrift_spec[k][1] for k in sorted(args.thrift_spec)]

        @asyncio.coroutine
        def call():
            f = getattr(self._handler, api)
            arguments = (args.__dict__[k] for k in api_args)
            if asyncio.iscoroutinefunction(f):
                rv = yield from f(*arguments)
                return rv
            return f(*arguments)

        return api, seqid, result, call

    def send_exception(self, oprot, api, exc, seqid):
        oprot.write_message_begin(api, TMessageType.EXCEPTION, seqid)
        exc.write(oprot)
        oprot.write_message_end()
        oprot.trans.drain()

    def send_result(self, oprot, api, result, seqid):
        oprot.write_message_begin(api, TMessageType.REPLY, seqid)
        result.write(oprot)
        oprot.write_message_end()
        oprot.trans.flush()

    def handle_exception(self, e, result):
        for k in sorted(result.thrift_spec):
            if result.thrift_spec[k][1] == "success":
                continue

            _, exc_name, exc_cls, _ = result.thrift_spec[k]
            if isinstance(e, exc_cls):
                setattr(result, exc_name, e)
                break
        else:
            raise

    @asyncio.coroutine
    def process(self, iprot, oprot):
        api, seqid, result, call = yield from self.process_in(iprot)

        if isinstance(result, TApplicationException):
            return self.send_exception(oprot, api, result, seqid)

        try:
            result.success = yield from call()
        except Exception as e:
            # raise if api don't have throws
            self.handle_exception(e, result)

        if not result.oneway:
            self.send_result(oprot, api, result, seqid)
